(******************************************************************************)
(* computer                                                                   *)
(* (C) 2010..2021 Steffen Solyga <solyga@gmx.de>                              *)
(******************************************************************************)
IMPLEMENTATION MODULE Comp;

IMPORT FIO;

CONST
  debug = FALSE;
  out ::= FIO.StandardOutput;
  err ::= FIO.ErrorOutput;
  dbg ::= FIO.ErrorOutput;
  lenMax = 16;
  idxMin = 1;

VAR
  stk: ARRAY [ idxMin .. lenMax ] OF TValue;
  len: CARDINAL;


PROCEDURE Err( s: ARRAY OF CHAR );
BEGIN
  FIO.WrStr( err, 'Comp: ' );
  FIO.WrStr( err, s );
  FIO.WrStr( err, '.' );
  FIO.WrLn( err );
  HALT;
END Err;

PROCEDURE Val( v: TValue );
BEGIN
  IF debug THEN
    FIO.WrStr( dbg, 'Comp.Debug: Val ' );
    FIO.WrInt( dbg, v, 0 );
    FIO.WrLn( dbg );
  END;
  INC( len );
  IF len > lenMax THEN Err( 'Stack overflow' ) END;
  stk[ len ] := v;
END Val;

PROCEDURE ChkMin( x: CARDINAL );
BEGIN
  IF len < x THEN Err( 'Stack underflow' ) END;
END ChkMin;

PROCEDURE Opr( o: TOperation );
BEGIN
  IF debug THEN
    FIO.WrStr( dbg, 'Comp.Debug: Opr ' );
    FIO.WrCard( dbg, ORD(o), 0 );
    FIO.WrLn( dbg );
  END;
  CASE o OF
  | oNeg: ChkMin(1); stk[len] := -stk[len];
  | oWrt: ChkMin(1); FIO.WrInt(out,stk[len],0); FIO.WrLn(out); DEC(len);
  | oAdd: ChkMin(2); DEC(len); stk[len] := stk[len] + stk[len+1];
  | oSub: ChkMin(2); DEC(len); stk[len] := stk[len] - stk[len+1];
  | oMul: ChkMin(2); DEC(len); stk[len] := stk[len] * stk[len+1];
  | oDiv: ChkMin(2); IF stk[len] = 0 THEN Err( 'Division by zero' ) END;
          DEC(len); stk[len] := stk[len] DIV stk[len+1];
  | oNew: INC(len);
  ELSE
    Err( 'Bug: Invalid operation' );
  END;
END Opr;

PROCEDURE Sto( o: TOffset );
BEGIN
  IF debug THEN
    FIO.WrStr( dbg, 'Comp.Debug: Sto ' );
    FIO.WrCard( dbg, o, 0 );
    FIO.WrLn( dbg );
  END;
  stk[ idxMin + o ] := stk[ len ]; DEC( len );
END Sto;

PROCEDURE Lod( o: TOffset );
BEGIN
  IF debug THEN
    FIO.WrStr( dbg, 'Comp.Debug: Lod ' );
    FIO.WrCard( dbg, o, 0 );
    FIO.WrLn( dbg );
  END;
  INC( len );
  IF len > lenMax THEN Err( 'Stack overflow' ) END;
  stk[ len ] := stk[ idxMin + o ];
END Lod;

BEGIN
  len := 0;
END Comp.
